package com.example.epl498_labs.ui.firebase;

import android.app.DatePickerDialog;
import android.os.Bundle;
import android.os.Handler;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;
import androidx.navigation.fragment.NavHostFragment;

import com.example.epl498_labs.R;
import com.example.epl498_labs.databinding.FragmentUserEditInfoBinding;
import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;
import com.google.firebase.firestore.DocumentReference;
import com.google.firebase.firestore.DocumentSnapshot;
import com.google.firebase.firestore.FirebaseFirestore;
import com.google.firebase.firestore.Source;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * This class is responsible for demonstrating on how to edit and delete firebase data
 */
public class UserEditInfoFragment extends Fragment {

    private FragmentUserEditInfoBinding binding;
    String documentReferenceId;
    private EditText editTextName, editTextSurname, editTextDescription, editTextBirthdate;
    private RadioGroup radioGroupFindUs;
    private Button buttonSubmit, buttonDelete;

    private FirebaseFirestore db;
    private DatePickerDialog datePickerDialog;
    private SimpleDateFormat dateFormat;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        // Retrieve the document reference ID from the arguments
        Bundle args = getArguments();
        if (args != null) {
            documentReferenceId = args.getString("documentReferenceId");
        }

        // Initialize Firebase Firestore
        db = FirebaseFirestore.getInstance();
        dateFormat = new SimpleDateFormat("dd-MM-yyyy");
    }

    @Override
    public View onCreateView(
            LayoutInflater inflater, ViewGroup container,
            Bundle savedInstanceState
    ) {
        // Inflate the binding for this fragment
        binding = FragmentUserEditInfoBinding.inflate(inflater, container, false);

        // Initialize UI elements
        editTextName = binding.editTextName;
        editTextSurname = binding.editTextSurname;
        editTextDescription = binding.editTextDescription;
        editTextBirthdate = binding.editTextBirthdate;
        radioGroupFindUs = binding.radioGroupFindUs;
        buttonSubmit = binding.buttonSubmit;
        buttonDelete = binding.buttonDelete;

        // Get a reference to the Firestore document based on the documentReferenceId
        DocumentReference docRef = db.collection("users").document(documentReferenceId);
        Source source = Source.DEFAULT;

        // Retrieve document data from Firestore
        docRef.get(source).addOnCompleteListener(new OnCompleteListener<DocumentSnapshot>() {
            @Override
            public void onComplete(@NonNull Task<DocumentSnapshot> task) {
                if (task.isSuccessful()) {
                    DocumentSnapshot document = task.getResult();
                    if (document.exists()) {
                        // Set the retrieved user data in the UI
                        setUserInfo(document);
                    } else {
                        Toast.makeText(requireContext(), "Document does not exist", Toast.LENGTH_LONG).show();
                    }
                } else {
                    Toast.makeText(requireContext(), "Failure: could not retrieve data...", Toast.LENGTH_LONG).show();
                }
            }
        });

        // Initialize the DatePickerDialog
        datePickerDialog = new DatePickerDialog(requireContext(), (view, year, month, dayOfMonth) -> {
            Calendar calendar = Calendar.getInstance();
            calendar.set(year, month, dayOfMonth);

            // Format the date
            String formattedDate = dateFormat.format(calendar.getTime());

            // Display the formatted date in the EditText
            editTextBirthdate.setText(formattedDate);
        }, Calendar.getInstance().get(Calendar.YEAR), Calendar.getInstance().get(Calendar.MONTH), Calendar.getInstance().get(Calendar.DAY_OF_MONTH));

        // Show date picker dialog when the birthdate EditText is clicked
        editTextBirthdate.setOnClickListener(v -> datePickerDialog.show());

        // Define a click listener for the "Save" button
        buttonSubmit.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                // Get user input
                String name = editTextName.getText().toString();
                String surname = editTextSurname.getText().toString();
                String description = editTextDescription.getText().toString();
                Date birthdate = null;
                try {
                    birthdate = dateFormat.parse(editTextBirthdate.getText().toString());
                } catch (ParseException e) {
                    throw new RuntimeException(e);
                }
                RadioButton radioButton = binding.getRoot().findViewById(radioGroupFindUs.getCheckedRadioButtonId());
                String howDidYouFindUs = (String) radioButton.getText();

                // Create a map to update user data in Firestore
                Map<String, Object> userUpdate = new HashMap<>();
                userUpdate.put("name", name);
                userUpdate.put("surname", surname);
                userUpdate.put("birthdate", birthdate);
                userUpdate.put("description", description);
                userUpdate.put("howDidYouFindUs", howDidYouFindUs);

                // Update the document with the new user data
                docRef.update(userUpdate)
                        .addOnSuccessListener(documentReference -> {
                            // Show a success message
                            Toast.makeText(requireContext(), "Info Saved Successfully", Toast.LENGTH_SHORT).show();

                            // Post a delayed action to navigate after 3 seconds
                            new Handler().postDelayed(new Runnable() {
                                @Override
                                public void run() {
                                    // Create a bundle to send the documentReferenceId to the next fragment
                                    Bundle bundle = new Bundle();
                                    bundle.putString("documentReferenceId", documentReferenceId);

                                    // Navigate to the UserViewInfoFragment with the bundle
                                    NavHostFragment.findNavController(UserEditInfoFragment.this)
                                            .navigate(R.id.action_UserEditInfoFragment_to_UserViewInfoFragment, bundle);
                                }
                            }, 3000); // 3000 milliseconds (3 seconds)
                        })
                        .addOnFailureListener(e -> {
                            // Show a failure message
                            Toast.makeText(requireContext(), "Failure: info were not saved...", Toast.LENGTH_LONG).show();
                        });
            }
        });

        // Define a click listener for the "Delete" button
        buttonDelete.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                // Delete the document from Firestore
                docRef.delete()
                        .addOnSuccessListener(documentReference -> {
                            // Show a success message
                            Toast.makeText(requireContext(), "Info Deleted Successfully", Toast.LENGTH_SHORT).show();

                            // Post a delayed action to navigate after 3 seconds
                            new Handler().postDelayed(new Runnable() {
                                @Override
                                public void run() {
                                    // Navigate to the FirebaseFragment
                                    NavHostFragment.findNavController(UserEditInfoFragment.this)
                                            .navigate(R.id.action_UserEditInfoFragment_to_FirebaseFragment);
                                }
                            }, 3000); // 3000 milliseconds (3 seconds)
                        })
                        .addOnFailureListener(e -> {
                            // Show a failure message
                            Toast.makeText(requireContext(), "Failure: info were not deleted...", Toast.LENGTH_LONG).show();
                        });
            }
        });

        return binding.getRoot();
    }

    // Helper method to set user information in the UI
    private void setUserInfo(DocumentSnapshot document) {
        String name = document.getString("name");
        String surname = document.getString("surname");
        String description = document.getString("description");
        Date birthdate = document.getDate("birthdate");
        String howDidYouFindUs = document.getString("howDidYouFindUs");

        // Set the user information in the UI
        editTextName.setText(name);
        editTextSurname.setText(surname);
        editTextDescription.setText(description);

        if (birthdate != null) {
            editTextBirthdate.setText(dateFormat.format(birthdate));
        }

        if (howDidYouFindUs != null) {
            switch (howDidYouFindUs) {
                case "Online":
                    radioGroupFindUs.check(binding.radioOnline.getId());
                    break;
                case "A friend":
                    radioGroupFindUs.check(binding.radioFriend.getId());
                    break;
                case "Other":
                    radioGroupFindUs.check(binding.radioOther.getId());
                    break;
            }
        }
    }
}